/*
  media-filesystem library
  tridge@samba.org, January 2001
  released under the Gnu GPL v2
*/

#include "mfs.h"
#ifndef TIVO			/* ppchacker, remove compiler warning */
#include <netinet/tcp.h>
#endif

void read_all(int fd, void *buf, int size)
{
	while (size) {
		int n = read(fd, buf, size);
		if (n <= 0) {
			// fprintf(stderr,"ERROR: eof in read_all\n");
			exit(1);
		}
		buf += n;
		size -= n;
	}
}

void write_all(int fd, void *buf, int size)
{
	while (size) {
		int n = write(fd, buf, size);
		if (n <= 0) {
			// fprintf(stderr,"ERROR: eof in write_all\n");
			exit(1);
		}
		buf += n;
		size -= n;
	}
}

/* open a socket to a tcp remote host with the specified port 
   based on code from Warren */
int open_socket_out(char *host, int port)
{
	int type = SOCK_STREAM;
	struct sockaddr_in sock_out;
	int res;
	struct hostent *hp;  
	int nodelay = 1;
	int window = 65535;

	res = socket(PF_INET, type, 0);
	if (res == -1) {
		return -1;
	}

	hp = gethostbyname(host);
	if (!hp) {
		fprintf(stderr,"unknown host: %s\n", host);
		return -1;
	}

	memcpy(&sock_out.sin_addr, hp->h_addr, hp->h_length);
	sock_out.sin_port = htons(port);
	sock_out.sin_family = PF_INET;

	if (connect(res,(struct sockaddr *)&sock_out,sizeof(sock_out))) {
		close(res);
		fprintf(stderr,"failed to connect to %s - %s\n", 
			host, strerror(errno));
		return -1;
	}

	setsockopt(res, SOL_SOCKET, SO_RCVBUF, &window, sizeof(int));
	setsockopt(res, IPPROTO_TCP, TCP_NODELAY, &nodelay, sizeof(int));

	return res;
}

void byte_swap(void *p, char *desc)
{
	int n, i;
	while (*desc) {
		switch (*desc) {
		case 'i': {
			u32 *v;
			n = strtol(desc+1, &desc, 10);
			v = p;
			for (i=0;i<n;i++) v[i] = ntohl(v[i]);
			p = (void *)(v+n);
			break;
		}

		case 's': {
			u16 *v;
			n = strtol(desc+1, &desc, 10);
			v = p;
			for (i=0;i<n;i++) v[i] = ntohs(v[i]);
			p = (void *)(v+n);
			break;
		}

		case 'b': {
			n = strtol(desc+1, &desc, 10);
			p += n;
			break;
		}
		}
		while (*desc == ' ') desc++;
	}
}


u64 llseek(int fd, u64 offset, int whence)
{
	u64 result;
	if (syscall(__NR__llseek, fd, (u32)(offset>>32), (u32)(offset&0xffffffff), &result, whence) != 0) {
		fprintf(stderr,"llseek failed\n");
		exit(1);
	}
	return result;
}

u32 read_sectors(int fd, void *buf, u32 sector, u32 count)
{
#ifdef TIVO
	struct FsIovec vec;
	struct FsIoRequest req;

	vec.pb = buf;
	vec.cb = count*SECTOR_SIZE;
	req.sector = sector;
	req.num_sectors = count;
	req.deadline = 0;
	
	return syscall(__NR_readsectors, fd, &vec, 1, &req);
#else
	llseek(fd, ((u64)sector)<<SECTOR_SHIFT, SEEK_SET);
	return read(fd, buf, count * SECTOR_SIZE)>>SECTOR_SHIFT;
#endif
}

u32 write_sectors(int fd, void *buf, u32 sector, u32 count)
{
#ifdef TIVO
	struct FsIovec vec;
	struct FsIoRequest req;

	vec.pb = buf;
	vec.cb = count*SECTOR_SIZE;
	req.sector = sector;
	req.num_sectors = count;
	req.deadline = 0;
	
	return syscall(__NR_writesectors, fd, &vec, 1, &req);
#else
	llseek(fd, ((u64)sector)<<SECTOR_SHIFT, SEEK_SET);
	return write(fd, buf, count * SECTOR_SIZE)>>SECTOR_SHIFT;
#endif
}


/****************************************************************************
Set a fd into nonblocking mode
****************************************************************************/
void set_nonblocking(int fd)
{
	int val;

	if((val = fcntl(fd, F_GETFL, 0)) == -1)
		return;
	if (!(val & O_NONBLOCK)) {
		val |= O_NONBLOCK;
		fcntl(fd, F_SETFL, val);
	}
}

int bitcount32(u32 x)
{
	int count;
	for (count=0; x; count++)
		x &= (x-1);
	return count;
}
